##-----------------------------LICENSE NOTICE------------------------------------
##  This file is part of CPCtelera: An Amstrad CPC Game Engine 
##  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
##
##  This program is free software: you can redistribute it and/or modify
##  it under the terms of the GNU Lesser General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public License
##  along with this program.  If not, see <http://www.gnu.org/licenses/>.
##------------------------------------------------------------------------------
############################################################################
##                        CPCTELERA ENGINE                                ##
##                 Automatic compression utilities                        ##
##------------------------------------------------------------------------##
## This file is intended for users to automate the generation of          ##
## compressed files and their inclusion in users' projects.               ##
############################################################################

## COMPRESSION EXAMPLE (Uncomment lines to use)
##

## First 3 calls to ADD2PACK add enemy, hero and background
## graphics (previously converted to binary data) into the 
## compressed group 'mygraphics'. After that, call to PACKZX7B
## compresses all the data and generates an array with the result
## that is placed in src/mygraphics.c & src/mygraphics.h, ready
## to be included and used by other modules.
##
#$(eval $(call ADD2PACK,mygraphics,gfx/enemy.bin))
#$(eval $(call ADD2PACK,mygraphics,gfx/hero.bin))
#$(eval $(call ADD2PACK,mygraphics,gfx/background.bin))
#$(eval $(call PACKZX7B,mygraphics,src/))

## Indicar todo lo que se quiere comprimir
$(eval $(call ADD2PACK,mapa1,src/maps/bin/mapa1.bin))
$(eval $(call ADD2PACK,mapa2,src/maps/bin/mapa2.bin))
$(eval $(call ADD2PACK,mapa3,src/maps/bin/mapa3.bin))
$(eval $(call ADD2PACK,mapa4,src/maps/bin/mapa4.bin))
$(eval $(call ADD2PACK,mapa5,src/maps/bin/mapa5.bin))

$(eval $(call ADD2PACK,cara_mago0,src/sprites/bin/cara_mago0.bin))
$(eval $(call ADD2PACK,jugador_mago0,src/sprites/bin/jugador_mago0.bin))

$(eval $(call ADD2PACK,cara_mago1,src/sprites/bin/cara_mago1.bin))
$(eval $(call ADD2PACK,jugador_mago1,src/sprites/bin/jugador_mago1.bin))

$(eval $(call ADD2PACK,cara_zombie,src/sprites/bin/cara_zombie.bin))
$(eval $(call ADD2PACK,jugador_zombie,src/sprites/bin/jugador_zombie.bin))

$(eval $(call ADD2PACK,cara_larcenas,src/sprites/bin/cara_larcenas.bin))
$(eval $(call ADD2PACK,jugador_larcenas,src/sprites/bin/jugador_larcenas.bin))

$(eval $(call ADD2PACK,cara_rana,src/sprites/bin/cara_rana.bin))
$(eval $(call ADD2PACK,jugador_rana,src/sprites/bin/jugador_rana.bin))

$(eval $(call ADD2PACK,cara_bruja,src/sprites/bin/cara_bruja.bin))
$(eval $(call ADD2PACK,jugador_bruja,src/sprites/bin/jugador_bruja.bin))

$(eval $(call ADD2PACK,logo,src/sprites/bin/logogame.bin))

## Llamar a la compresión
$(eval $(call PACKZX7B,mapa1,src/maps/))
$(eval $(call PACKZX7B,mapa2,src/maps/))
$(eval $(call PACKZX7B,mapa3,src/maps/))
$(eval $(call PACKZX7B,mapa4,src/maps/))
$(eval $(call PACKZX7B,mapa5,src/maps/))

$(eval $(call PACKZX7B,cara_mago0,src/sprites/))
$(eval $(call PACKZX7B,cara_mago1,src/sprites/))
$(eval $(call PACKZX7B,cara_zombie,src/sprites/))
$(eval $(call PACKZX7B,cara_rana,src/sprites/))
$(eval $(call PACKZX7B,cara_larcenas,src/sprites/))
$(eval $(call PACKZX7B,cara_bruja,src/sprites/))

$(eval $(call PACKZX7B,jugador_mago0,src/sprites/))
$(eval $(call PACKZX7B,jugador_mago1,src/sprites/))
$(eval $(call PACKZX7B,jugador_zombie,src/sprites/))
$(eval $(call PACKZX7B,jugador_rana,src/sprites/))
$(eval $(call PACKZX7B,jugador_larcenas,src/sprites/))
$(eval $(call PACKZX7B,jugador_bruja,src/sprites/))

$(eval $(call PACKZX7B,logo,src/sprites/))


############################################################################
##              DETAILED INSTRUCTIONS AND PARAMETERS                      ##
##------------------------------------------------------------------------##
##                                                                        ##
## Macros used for compression are ADD2PACK and PACKZX7B:                 ##
##                                                                        ##
##	ADD2PACK: Adds files to packed (compressed) groups. Each call to this ##
##  		  macro will add a file to a named compressed group.          ##
##  PACKZX7B: Compresses all files in a group into a single binary and    ##
##            generates a C-array and a header to comfortably use it from ##
##            inside your code.                                           ##
##                                                                        ##
##------------------------------------------------------------------------##
##                                                                        ##
##  $(eval $(call ADD2PACK,<packname>,<file>))                            ##
##                                                                        ##
##		Sequentially adds <file> to compressed group <packname>. Each     ##
## call to this macro adds a new <file> after the latest one added.       ##
## packname could be any valid C identifier.                              ##
##                                                                        ##
##  Parameters:                                                           ##
##  (packname): Name of the compressed group where the file will be added ##
##  (file)    : File to be added at the end of the compressed group       ##
##                                                                        ##
##------------------------------------------------------------------------##
##                                                                        ##
##  $(eval $(call PACKZX7B,<packname>,<dest_path>))                       ##
##                                                                        ##
##		Compresses all files in the <packname> group using ZX7B algorithm ##
## and generates 2 files: <packname>.c and <packname>.h that contain a    ##
## C-array with the compressed data and a header file for declarations.   ##
## Generated files are moved to the folder <dest_path>.                   ##
##                                                                        ##
##  Parameters:                                                           ##
##  (packname) : Name of the compressed group to use for packing          ##
##  (dest_path): Destination path for generated output files              ##
##                                                                        ##
############################################################################
##                                                                        ##
## Important:                                                             ##
##  * Do NOT separate macro parameters with spaces, blanks or other chars.##
##    ANY character you put into a macro parameter will be passed to the  ##
##    macro. Therefore ...,src/sprites,... will represent "src/sprites"   ##
##    folder, whereas ...,  src/sprites,... means "  src/sprites" folder. ##
##  * You can omit parameters by leaving them empty.                      ##
##  * Parameters (4) and (5) are optional and generally not required.     ##
############################################################################
