#include <cpctelera.h>
#include "main.h"
#include "font_utils.h"
#include "fonts/font4x5.h"
#include "string.h"


// ==============================================================
// IMPRIMIR UNA CADENA DE TEXTO CON LA FUENTE DE SPRITES
//        Parámetros:
//              - u8 frase      - Linea de texto a escribir
//              - u8 x          - Coordenada X (Si es '0' centrará el texto automáticamente)
//              - u8 y          - Coordenada Y
//              - transparente  - SI = Sprites transparentes / NO = Sprites con fondo negro
//              - pen           - Si es 15 dibujará en blanco
// ==============================================================
void printTxt(u8 frase[25], u8 x, u8 y, u8 transparente, u8 pen) {
        
    // SOLO UTILIZAR LETRAS MAYÚSCULAS O SALDRÁN CARACTERES RAROS
    //
    // Sólo se hará el cambio de PEN cuando el texto NO es transparente
    // 
    u8 i, character;
    u8* vmem = VMEM_BUFFER1; // Si no se indica el doble bufer sólo lo escribirá en la memoria de pantalla estandar
    u8* vmem_pos;
    u16 replacePatColor;

    // Si la 'x' es cero centramos el texto en pantalla (40 es la mitad de ancho de pantalla del modo 0)
    if (x == CENTRADO) x = 40 - strlen(frase);

    /*
    if (x == CENTRADO) {
        while (frase[character] != '\0') character++;
        x = 40 - character; // 40 es la mitad de la pantalla en modo 0
    }
    */

    for (i = NUM_BUFFERS; i > 0; i--) {
        character = 0;
        vmem_pos = cpct_getScreenPtr(vmem, x, y);
        do {
            if (frase[character] == 32) frase[character] = 64; // Sustituimos los espacios por arrobas, que en el sprite de fuentes corresponden al espacio
            if (!pen) {
                if (!transparente) {
                    cpct_drawSprite(sp_font + (frase[character] - PRIMER_CARACTER_ASCII) * FONT_ALTO_POR_2,
                        vmem_pos + (character * FONT_ANCHO), FONT_ANCHO, FONT_ALTO);
                } else {
                    cpct_drawSpriteMaskedAlignedTable(sp_font + (frase[character] - PRIMER_CARACTER_ASCII) * FONT_ALTO_POR_2,
                        vmem_pos + (character * FONT_ANCHO),
                        FONT_ANCHO, FONT_ALTO, sp_tablaTrans);
                }
            } else {
                replacePatColor = cpct_pens2pixelPatternPairM0(15, pen);
                cpct_drawSpriteColorizeM0(sp_font + (frase[character] - PRIMER_CARACTER_ASCII) * FONT_ALTO_POR_2, 
                        vmem_pos + (character * FONT_ANCHO), FONT_ANCHO, FONT_ALTO, replacePatColor);
            }
            character++;
        }  while (frase[character]!='\0');

        vmem = VMEM_BUFFER2;
    }
}

// ==============================================
// DIBUJAR UNA LINEA AL ESTILO SUBRAYADO
// 
// - RECIBE:
//     x:          Coordenada X donde comenzar a subrayar
//     y:          Coordenada Y
//     numChars:   Núnero de caracteres a escribir (longitud del subrayado)
// ==============================================
void subrayar(u8 x, u8 y, u8 numChars) {
    u8 i;
    for (i = 0; i < numChars; i++) {
        printTxt(",", x + i, y, SI, 10);
    }
}

// ==============================================================
// IMPRIMIR UN NÚMERO CON LA FUENTE DE SPRITES
//        Parámetros:
//              - u8 num        - Enero a escribir
//              - u8 x          - Coordenada X (Si es '0' o 'CENTRADO' centrará el texto automáticamente)
//              - u8 y          - Coordenada Y
//              - transparente  - SI = Sprites transparentes / NO = Sprites con fondo negro
//              - pen           - Si es 15 dibujará en blanco
// ==============================================================
void printInt(u16 num, u8 x, u8 y, u8 transparente, u8 pen) {
    
    // Cuando lo que queremos escribir en pantalla es un número
    // primero tenemos que convertirlo a string y se lo enviamos a la función de pintar strings
    //
    u8 frase[6];
    Itoa(num, frase);
    printTxt(frase, x, y, transparente, pen);
}


/*-------------------------------------------------------------------------
 integer to string conversion
 Written by:   Bela Torok, 1999
               bela.torok@kssg.ch
 usage:
 uitoa(unsigned int value, char* string, int radix)
 itoa(int value, char* string, int radix)
 value  ->  Number to be converted
 string ->  Result
 radix  ->  Base of value (e.g.: 2 for binary, 10 for decimal, 16 for hex)
---------------------------------------------------------------------------*/
#define RADIX 10

void Itoa(u16 value, char* string) {
    u8 index, i;
    index = 5;
    i = 0;
    do {
        string[--index] = '0' + (value % RADIX);
        if ( string[index] > '9') string[index] += 'A' - ':';   /* continue with A, B,.. */
        value /= RADIX;
    } while (value != 0);

    do {
        string[i++] = string[index++];
    } while ( index < 5 );

    string[i] = 0; /* string terminator */ 
}


/////////////////////////////////////////////
// OTRA FUNCION ITOA:
/////////////////////////////////////////////
//
/*

  // Itoa --> converts an integer to ASCII (Lukás Chmela / GPLv3)
	 * C++ version 0.4 char* style "itoa":
	 * Written by Lukás Chmela
	 * Released under GPLv3.
     * https://gist.github.com/parzibyte/5f1be7863a2130cd10505f4343f36c1b


char *itoa(int value, char *result, int base) {
     // check that the base if valid
     if (base < 2 || base > 36) {
          *result = '\0';
          return result;
     }

     char *ptr = result, *ptr1 = result, tmp_char;
     int tmp_value;

     do {
          tmp_value = value;
          value /= base;
          *ptr++ = "zyxwvutsrqponmlkjihgfedcba9876543210123456789abcdefghijklmnopqrstuvwxyz"[35 +
                                                                                               (tmp_value - value * base)];
     } while (value);

     // Apply negative sign
     if (tmp_value < 0) *ptr++ = '-';
     *ptr-- = '\0';
     while (ptr1 < ptr) {
          tmp_char = *ptr;
          *ptr-- = *ptr1;
          *ptr1++ = tmp_char;
     }
     return result;
}
*/


////////////////////////////////////////////////////////////////////////
// LAS MISMAS FUNCIONES PERO CON OPCION A MONOBUFER. 
// Comento éstas y fuerzo el doble búfer para ahorrar algo de espacio
////////////////////////////////////////////////////////////////////////
//
/*
void printTxt(u8 frase[25],u8 x,u8 y, u8 transparente, u8 monoBufer, u8 pen) {
        
    // SOLO UTILIZAR LETRAS MAYÚSCULAS O SALDRÁN CARACTERES RAROS
    //
    // Sólo se puede hacer el cambio de PEN cuando el texto NO es transparente
    // 

    u8 i, character;
    u8* vmem = VMEM_BUFFER1; // Si no se indica el doble bufer sólo lo escribirá en la memoria de pantalla estandar
    u8* vmem_pos;
    u16 replacePatColor;
    character = 0;

    // Si la 'x' es cero centramos el texto en pantalla (40 es la mitad de ancho de pantalla del modo 0)
    if (x == 0) {
        while (frase[character] != '\0') {
            character++;
        }
        x = 40 - character;
    }

    for (i = monoBufer; i < NUM_BUFFERS; i++) {
        //i = i + monoBufer;
        character = 0;
        vmem_pos = cpct_getScreenPtr(vmem, x, y);
        do {
            if (frase[character] == 32) frase[character] = 64; // Sustituimos los espacios por arrobas, que en el sprite de fuentes corresponden al espacio
            if (pen == 15) {
                if (!transparente) {
                    cpct_drawSprite(sp_font + (frase[character] - PRIMER_CARACTER_ASCII) * FONT_ALTO_POR_2,
                        vmem_pos + (character * FONT_ANCHO), FONT_ANCHO, FONT_ALTO);
                } else {
                    cpct_drawSpriteMaskedAlignedTable(sp_font + (frase[character] - PRIMER_CARACTER_ASCII) * FONT_ALTO_POR_2,
                        vmem_pos + (character * FONT_ANCHO),
                        FONT_ANCHO, FONT_ALTO, sp_tablaTrans);
                }
            } else {
                replacePatColor = cpct_pens2pixelPatternPairM0(15, pen);
                cpct_drawSpriteColorizeM0(sp_font + (frase[character] - PRIMER_CARACTER_ASCII) * FONT_ALTO_POR_2, 
                        vmem_pos + (character * FONT_ANCHO), FONT_ANCHO, FONT_ALTO, replacePatColor);
            }
            character++;
        } while (frase[character]!='\0');

        vmem = VMEM_BUFFER2;
    }
}

void printInt(u16 num, u8 x, u8 y, u8 transparente, u8 monoBufer, u8 color) {
    
    // Cuando lo que queremos escribir en pantalla es un número
    // primero tenemos que convertirlo a string y se lo enviamos a la función de pintar strings
    //
    u8 frase[6];
    Itoa(num, frase);
    printTxt(frase, x, y, transparente, monoBufer, color);
}
*/
