#include <cpctelera.h>
#include "main.h"
#include "maps/mapa1.h"
#include "maps/mapa2.h"
#include "maps/mapa3.h"
#include "maps/mapa4.h"
#include "maps/mapa5.h"
#include "tiles/tiles8x8.h"
#include "render.h"
#include "font_utils.h"
#include "gui.h"

const u8 map_names[NUM_SELECT_STAGES][11] = {
    "SPELLBOUND"
,   "HOCUSPOCVS"
,   "   ANKH   "
,   "SORTILEGE "
,   " ALACAZAM "
,   " BE LUCKY "
};


// ==============================================================
// ACTUALIZA EL PUNTERO "mapa_ptr" AL MAPA CORRESPONDIENTE
//
// - PARÁMETROS:
//      pantallaDemo:   NO -> Coloca el puntero en el mapa definido por ' mapa_actual '
//                      SI -> Coloca el puntero en un mapa al azar
//
// - REQUISITOS: 
//      Debe estar definida la variable "mapa_actual" con
//      el número de mapa del que se quiere actualizar el puntero
//
// - NOTA:  Si la variable global ' selectedStage ' es = 6 (el mapa random)
//          también colocará el puntero en un mapa al azar
// ==============================================================
void punteroMapa(u8 pantallaDemo) {
    
    u8 mapa_anterior = mapa_actual;
    
    if (selectedStage == NUM_MAPA_RANDOM || pantallaDemo) {
        do {
            mapa_actual = (cpct_rand() % NUM_MAPAS) + 1;
        } while (mapa_actual == mapa_anterior);
    } else {
        mapa_actual = selectedStage;
    }

    // Con el IF a cada mapa ocupa menos memoria que con el switch
    //
    mapa_ptr = (u8*) mapa1_end;
    if (mapa_actual == 2) mapa_ptr = (u8*) mapa2_end;
    if (mapa_actual == 3) mapa_ptr = (u8*) mapa3_end;
    if (mapa_actual == 4) mapa_ptr = (u8*) mapa4_end;
    if (mapa_actual == 5) mapa_ptr = (u8*) mapa5_end;
}


// ==============================================================
// PINTA EL MAPA ACTUAL EN PANTALLA
//
// - REQUISITOS:
//      El mapa estará descomprimido en el array ' mapa[] '
// ==============================================================
void pintarMapa() {
    u8 j,i;
    //u8 numTiles = 2;
    u8* vmem_pos;
    u8* vmem_buffer = VMEM_BUFFER1;


    for (j = NUM_BUFFERS; j > 0; j--) {
        // DIBUJAMOS EL MAPA EN LOS 2 BÚFFERS DE VIDEO
        //
        cpct_etm_setDrawTilemap4x8_ag(2, MAPA_H, MAPA_W, &tl_tileset_00[0]);
        // Establecemos el tileset
        for (i = 0; i < 20; i = i + 2) {
            vmem_pos = cpct_getScreenPtr(vmem_buffer, AREA_JUEGO_IZQ + (i*4), AREA_JUEGO_SUP);
            cpct_etm_drawTilemap4x8_ag(vmem_pos, &mapa[i]);
        }
        vmem_buffer = VMEM_BUFFER2;
    }
}

void escribirNombreMapa() {
    printTxt(map_names[selectedStage - 1], CENTRADO, 18, NO, 14);
}

// ==============================================================
// CREAR Y PINTA EL COLLAGE DE LA VISTA PREVIA DEL MAPA RANDOM
// PARA ELLO OBTIENE UN TROZO DE CADA MAPA Y DIBUJA LAS LINEAS DIVISORIAS VERTICALES
// ==============================================================
void pintarMapaRandom() {
    u8 j, i;
    u8* vmem_pos;
    u8* vmem_buffer;

    for (i = 1; i < NUM_MAPAS + 1; i++) {
        if (i == 1) mapa_ptr = (u8*) mapa1_end;
        if (i == 2) mapa_ptr = (u8*) mapa2_end;
        if (i == 3) mapa_ptr = (u8*) mapa3_end;
        if (i == 4) mapa_ptr = (u8*) mapa4_end;
        if (i == 5) mapa_ptr = (u8*) mapa5_end;
        cpct_zx7b_decrunch_s((void *) (&mapa[0] + TILES_TOTALES - 1), (void *) mapa_ptr);

        vmem_buffer = VMEM_BUFFER1;
        for (j = NUM_BUFFERS; j > 0; j--) {
            cpct_etm_setDrawTilemap4x8_ag(4, MAPA_H, MAPA_W, &tl_tileset_00[0]);
            vmem_pos = cpct_getScreenPtr(vmem_buffer, AREA_JUEGO_IZQ + ((i - 1) * 16), AREA_JUEGO_SUP);
            cpct_etm_drawTilemap4x8_ag(vmem_pos, &mapa[(i - 1) * 4]);
            if (i > 0 && i < 5) {
                vmem_pos = cpct_getScreenPtr(vmem_buffer, AREA_JUEGO_IZQ + (i * 16) - 1, AREA_JUEGO_SUP + 10);
                cpct_drawSolidBox(vmem_pos, 0xFF, 1, AREA_JUEGO_INF-AREA_JUEGO_SUP + 2 - 10);
            }
            vmem_buffer = VMEM_BUFFER2;
       }
    }
}

// ==============================================================
// DESCOMPRIME UN MAPA EN EL ARRAY ( mapa[] )
// Y DESPUÉS LLAMA A LA FUNCIÓN DE PINTARLO
//
// - REQUISITOS:
//      El puntero ' mapa_ptr ' debe apuntar al final de un mapa comprimido
// ==============================================================
void descomprimir_pintarMapa() {
    cpct_zx7b_decrunch_s((void *) (&mapa[0] + TILES_TOTALES - 1), (void *) mapa_ptr);
    pintarMapa();
}


// ==============================================================
// REDIBUJAR UN CONJUNTO DE TILES DEL MAPA ACTUAL
//  
// - PARÁMETROS:
//      x:          Coordenada X de inicio para el redibujado de tiles
//                  Se refiere a la coordenada en pantalla, no al num de tile
//      y:          Coordenada Y de inicio para el redibujado de tiles
//                  Se refiere a la coordenada en pantalla, no al num de tile
//      ancho:      Cantidad de tiles de ancho a redibujar
//      alto:       Cantidad de tiles de alto a redibujar
//      monoBufer:  Puede ser SI o NO
//                  SI -> Redibuja sólo 1 vez en el búfer activo (se utiliza para los tiles durante el gameplay)
//                  NO -> Redibuja 2 veces, una en cada bufer de video (se utiliza para los switches)
// ==============================================================
void redibujarTile(u8 x, u8 y, u8 ancho, u8 alto, u8 monoBufer) {
    u16 primer_tile;

    // RECOLOCAMOS LA X Y LA Y AL INICIO DE UN TILE
    //
    x = x - (x & 3) - AREA_JUEGO_IZQ;   // x = x - (x % TL_TILESET_00_W) - AREA_JUEGO_IZQ;
    y = y - (y & 7) - AREA_JUEGO_SUP;   // y = y - (y % TL_TILESET_00_H) - AREA_JUEGO_SUP;

    // Definir el primer tilemap a dibujar (son 20 tiles por fila)
    //
    primer_tile = (y / TL_TILESET_00_H) * 20 + (x / TL_TILESET_00_W);
    cpct_etm_setDrawTilemap4x8_ag (ancho, alto, 20, &tl_tileset_00[0]);
    if (monoBufer) {
        cpct_etm_drawTilemap4x8_ag((u8*) cpct_getScreenPtr (vmem_start, x + AREA_JUEGO_IZQ, y + AREA_JUEGO_SUP), &mapa[primer_tile]);
    } else {
        cpct_etm_drawTilemap4x8_ag((u8*) cpct_getScreenPtr (VMEM_BUFFER1, x + AREA_JUEGO_IZQ, y + AREA_JUEGO_SUP), &mapa[primer_tile]);
        cpct_etm_drawTilemap4x8_ag((u8*) cpct_getScreenPtr (VMEM_BUFFER2, x + AREA_JUEGO_IZQ, y + AREA_JUEGO_SUP), &mapa[primer_tile]);
    }
}

// ==============================================================
// REDIBUJAR LOS SWITCHES EN LOS MAPAS QUE DISPONEN DE ELLOS
//  - MAPAS QUE TIENEN SWITCHES -> 2 y 3
//      - Coordenadas de los switches
//  MAPA    X   Y
//  ----   --- ---
//  (2)     4,  72
//  (2)     4, 168
//  (3)    20, 104
//  (3)    56, 104
// ==============================================================
void redibujarSwitches() {
    if (mapa_actual == 3) {
        redibujarTile( 4,  72,  1,  1, NO);
        redibujarTile( 4, 168,  1,  1, NO);
    } else if (mapa_actual == 2) {
        redibujarTile(20,  104, 1,  1, NO);
        redibujarTile(56,  104, 1,  1, NO);
    }
}

// ==============================================================
// REALIZAR LOS CAMBIOS EN EL MAPA CUANDO SE PULSA UN SWITCH
// ==============================================================
void activarSwitch() {
    u8 n, n2;
    if (!mapa_halt_switch) {
        mapa_halt_switch = 500;
        
        ///////////////////////////////////////
        // REALIZAR LOS CAMBIOS EN EL MAPA
        ///////////////////////////////////////
        mapa_id_cambio = (mapa_id_cambio ? 0 : 1);
        // MAPA 3 - PYRAMID
        if (mapa_actual == 3) {
            // Desactivar los switches
            mapa[61] = mapa [301] = NUM_TILE_SWITCH_OFF;
            
            // Cambiar los obstáculos de sitio
            n = (mapa_id_cambio ? 2 : 25);
            n2 = (mapa_id_cambio ? 25 : 2);
            mapa[146] = mapa[266] = n;
            mapa[206] = mapa[326] = n2;
        } else        
        // MAPA 3 - HOCUSPOCVS
        if (mapa_actual == 2) {
            // Desactivar los switches
            mapa[145] = mapa [154] = NUM_TILE_SWITCH_OFF;
            
            // Cambiar los obstáculos de sitio
            n = (mapa_id_cambio ? 2 : 24);
            mapa[188] = mapa[189] = mapa[190] = mapa[191] = mapa[107] = mapa[108] = mapa[109] = mapa[110] = mapa[111] = mapa[112] = n;
        }

        ///////////////////////////////////////
        // DIBUJAR LOS CAMBIOS DEL MAPA EN PANTALLA
        ///////////////////////////////////////

        // MAPA 3
        if (mapa_actual == 3) {
            for (n = 104; n < 178; n = n + 24) {
                redibujarTile(24,  n,  1,  1, NO);
            }
        } else
        if (mapa_actual == 2) {
            redibujarTile(32,  120,  4,  1, NO);
            redibujarTile(28,   88,  6,  1, NO);
        }
        redibujarSwitches();
        playSFX(1,25);
    }
}


// ==============================================================
// VOLVER A HACER QUE LOS SWITCHES DEL NIVEL ESTÉN ACTIVOS
// ==============================================================
void reactivarSwitch() {
    mapa_halt_switch = 0;
    if (mapa_actual == 3) {
        mapa[61] = mapa [301] = NUM_TILE_SWITCH_ON;
    } else if (mapa_actual == 2) {
        mapa[145] = mapa [154] = NUM_TILE_SWITCH_ON;
    }
    redibujarSwitches();
}


// ==============================================================
// DEVUELVE LA POSICIÓN DE UN TILE DENTRO DEL ARRAY ' mapa[] ' 
//
// - PARÁMETROS:
//      x:  La coordenada X en pantalla de la que queremos obtener el num de tile
//      y:  La coordenada Y en pantalla de la que queremos obtener el num de tile
//
// - DEVUELVE:
//      u16:    La posición de esas coordenadas dentro del array ' mapa[] '
// ==============================================================
u16 tileCell(u8 x, u8 y) {
    return ((MAPA_W * ( (y - AREA_JUEGO_SUP) / TL_TILESET_00_H ) ) + ((x - AREA_JUEGO_IZQ) / TL_TILESET_00_W));
}

// ==============================================================
// FUNCIÓN QUE NOS DEVUELVE EL ID DE TILE DE UNA POSICIÓN
// EN CONCRETO DEL ARRAU ' mapa[] ' 
//
// - PARÁMETROS:
//      x:  La coordenada X en pantalla de la que queremos obtener el num de tile
//      y:  La coordenada Y en pantalla de la que queremos obtener el num de tile
//
// - DEVUELVE:
//      u8: El id de tile en las coordenadas facilitadas (el num de tile dentro del sprite tileset)
// ==============================================================
u8 tileValue(u8 x, u8 y) {
    return mapa[tileCell(x, y)];
}


// ==============================================================
// RECIBIENDO UN ID DE TILE, NOS DICE DE QUÉ TIPO ES
//
// - PARÁMETROS:
//      tile: El id de tile (el num de tile dentro del sprite tileset)
//
// - DEVUELVE:
//      u8: El tipo de tile, estos son:
//             · OBSTACULO:  Si es un tile con el que se produce colisión y no deja avanzar al jugador ni hechizos
//             · PLATAFORMA: El jugador al pisarlo no caerá. Los hechizos las pueden atravesar
//             · ESCALERAS:  El jugador al estar en contacto no cae, puede subir y bajar. Los hechizos las pueden atravesar
//             · PINCHOS:    Al entrar en contacto con el jugador activa las acciones de pincharse, los hechizos las pueden atravesar
//             · NUM_TILE_SWITCH:    Activa las acciones de los switches
//             · NADA:  Tile de decorado sin ninguna acción
//
// - REQUISITOS:
//      Los tiles deben seguir un orden concreto dentro del sprite de tileset para
//      poder reconocerlos
// ==============================================================
u8 typeOfTile(u8 tile) {
    if (tile == NUM_TILE_SWITCH) activarSwitch();
    if (tile > PRIMER_TILE_OBSTACULO)
        return OBSTACULO;
    else if (tile > PRIMER_TILE_PLATAFORMA)
        return PLATAFORMA;
    else if (tile > PRIMER_TILE_ESCALERA)
        return ESCALERAS;
    else if (tile > PRIMER_TILE_PINCHOS)
        return PINCHOS;
    else
        return NADA;
}