#include <cpctelera.h>
#include "main.h"
#include "render.h"
#include "teclado.h"
#include "gui.h"
#include "instrucciones.h"
#include "memory.h"
#include "font_utils.h"
#include "map_utils.h"
#include "maps/mapa1.h"
#include "maps/mapa2.h"
#include "maps/mapa3.h"
#include "maps/mapa4.h"
#include "maps/mapa5.h"
#include "tiles/tiles8x8.h"
#include "sprites/jugador_mago0.h"
#include "sprites/bin/jugador_mago0.h"
#include "sprites/bin/logogame.h"
#include "sprites/logo.h"
#include "sprites/arrow_left.h"
#include "audio/m_ingame.h"
#include "audio/m_sfx.h"
#include "audio/m_gameover.h"
#include "audio/m_bewitch.h"

// ==============================================================
// Definir los tiempos para mostrar los créditos
// y velocidad del sprite que camina borrándolos
// ==============================================================
#define NUM_CREDITOS        3
#define CAMBIAR_CADA     3500
#define PAUSA_CAMINANDO   55

// ==============================================================
// Fijamos la posición en memoria de array que almacena el logo descomprimido
// y lo situamos en el VMEM_BUFFER2 que no será visible en los menús
// ==============================================================
__at (VMEM_BUFFER2) const u8 logo_temp[logo_size];

// ==============================================================
// ESCRIBIR LOS CRÉDITOS
//
// - PARÁMETROS:
//      n:  El número que indica qué créditos queremos escribir,
//          definidos en el array de ' creditos[][] '
//
// - DEVUELVE:
//      u8: El número de indice de créditos sumándole uno (++n)
//          Si 'n' ha llegado al final de los créditos, devuelve 0
// ==============================================================
u8 escribirCreditos (u8 n){
    u8 i;
    u8 creditos[6][18] = {
            "-CODE ; GRAPHICS-"
        ,   "HINAMAN <2022>"
        ,   "-MUSIC ; TEST-"
        ,   "PIOJ"
        ,   "-TRANSLATIONS-"
        ,   "SUSANA TAPIA"
    };

    cpct_drawSolidBox(cpct_getScreenPtr((u8*) VMEM_BUFFER1, 13, 168), 0, 54, 18); 
    for (i = 0; i < 2; i++)
        printTxt(creditos[(n*2)+i], CENTRADO, 170+(i * 10), NO, 10);
    if (++n == NUM_CREDITOS) n = 0;
    return n;
}


// ==============================================================
// Función para descomprimir el logo en el array 'logo_temp[]'
// y pintarlo en pantalla.
// La primera vez que se muestra el logo se activa en 'bitOptions'
// el bit 'mask_first_run', y a partir de la segunda vez que se
// muestra el logo se mostrará con el color central (rosa) cambiado
// aleatoriamente entre el rosa inicial (10), el naranja (9) o el verde (11)
//
// - PARÁMETROS:
//      y:  La coordenada Y en donde se dibujará el logo (la coordenada X está fijada 
//          para mostrarse siempre centrado)
// ==============================================================
void pintaLogo(u8 y) {
    u16 replaceColors;
    u8* vmem_pos;
    u8 color = 10;

    // COLORES
    // 10 - Rosa
    //  9 - Naranja
    //  4 - Verde oscuro
    //  5 - Verde azuloso
    // 11 - Verde claro
    // 12 - Verde fosforito

    cpct_zx7b_decrunch_s((void *) (&logo_temp[0] + SP_LOGO_SIZE - 1), (u8*) logo_end);
    
    if (bitOptions & mask_first_run && (cpct_rand() & 0b00000010)) color = (cpct_rand() & 0x02 ? 9 : 11); // 'cpct_rand() & 0x02' es lo mismo que 'cpct_rand() % 2'
    replaceColors = cpct_pens2pixelPatternPairM0(10, color);
    vmem_pos = cpct_getScreenPtr(VMEM_BUFFER1, 25, y);
    cpct_drawSpriteMaskedAlignedColorizeM0 (logo_temp, vmem_pos, SP_LOGO_W, SP_LOGO_H, sp_tablaTrans, replaceColors);
    if (!(bitOptions & mask_first_run)) bitOptions = bitOptions | mask_first_run;
}

// ==============================================================
// Reproducir el sonido de tecla pulsada
// ==============================================================
void sonidoTecla() {
    playSFX(5,47);
}


// ==============================================================
// Muestra una imagen de un mapa aleatorio un tiempo o hasta que se pulse una tecla
// También muestra la cabecera de muro de ladrillos con el logo del juego encima
// ==============================================================
void demo() {
    u16 i = 0;

    borraPantallas();
    pintarMuro(0, 0, 4, 20);
    pintaLogo(0);

    mapa_actual = (cpct_rand() % NUM_MAPAS) + 1;
    punteroMapa(SI);
    descomprimir_pintarMapa();

    do {
        cpct_scanKeyboard();
        i++;
    } while (!cpct_isAnyKeyPressed() && i < 15000);
    mapa_actual = 0;
}


// ==============================================================
// Escribe los textos del menú de opciones
// teniendo en cuenta las opciones que están
// seleccionadas en ese momento
// ==============================================================
void pintaMenuOpciones() {
    u8 pen = 0;
    u8 x = 25;
    u8 y = 90;
    u8 optionTxt[17];

    //cpct_drawSolidBox(cpct_getScreenPtr(VMEM_BUFFER1, 33, 110), 0, 33, 5);

    //////////////////////////////////////////////////
    // ESCRIBIR LAS OPCIONES DEL MENÚ
    //
        // (1) - INSTRUCTIONS
        printTxt("1. INSTRUCTIONS", x, y, NO, 0);
        
        // (2) - REDEFINE
        printTxt("2. REDEFINE KEYS", x, y + 10, NO, 0);

        // (3) - MUSIC 'N SFX
        if (bitOptions & mask_music_on) {
            if (bitOptions & mask_sfx_on)   cpct_memcpy (optionTxt, "3. MUSIC ; FX\0", 16);
            else                            cpct_memcpy (optionTxt, "3. MUSIC ONLY\0", 16);
        } else {
            if (bitOptions & mask_sfx_on)   cpct_memcpy (optionTxt, "3. FX ONLY   \0", 16);
            else                            cpct_memcpy (optionTxt, "3. NO SOUND  \0", 16);
        }
        printTxt(optionTxt, x, y + 20, NO, 0);

        // (4) - GAME MODE
        if (bitOptions & mask_enable_countdown)     cpct_memcpy (optionTxt, "4. TIME ON \0", 12);
        else                                        cpct_memcpy (optionTxt, "4. TIME OFF\0", 12);
        printTxt(optionTxt, x, y + 30, NO, 0);

        // (5) - START GAME
        printTxt("5. START GAME", x, y + 40, NO, 0);
    
    vaciaBufferTeclado();
}


// ==============================================================
// Dirije el esquema de pintado del menú
// ==============================================================
void pintaMenu() {
    borraPantallas();
    pintarMuro(17, 13, 4, 11);
    pintaLogo(13);
    pintaMenuOpciones();
}

// ==============================================================
// Cambia el color del sprite de las flechas
// de la pantalla de selección de escenario
// para crear el efecto de pulsado
//
// - PARÁMETROS:
//      u8 c1 - Color a reemplazar (EL PEN)
//      u8 c2 - Nuevo color que reemplazará al c1 (EL PEN)
// ==============================================================
void cambiaColorFlecha(u8 c1, u8 c2) {
    cpct_spriteColourizeM0 (cpct_pens2pixelPatternPairM0 (c1, c2), 5 * 12, sp_arrow_left);
}

void giraSpriteFlecha(u8 girada) {
    if (girada) cpct_hflipSpriteM0(5, 12, sp_arrow_left);
}

// ==============================================================
// Dibuja la flecha de izquierda o derecha del menú selector de escenario
//
// - RECIBE:
//  u8 flecha       - 0  -> Posición izquierda | 1 -> Posición derecha
//  u8 girada       - NO -> Apunta a la izquierda | SI -> Apunta a la derecha
//  u8 iluminada    - NO -> Colores originales del sprite | SI -> Cambia los colores para el efecto de 'PULSADO'
// ==============================================================
void pintarFlecha(u8 flecha, u8 girada, u8 iluminada) {

    u8 x = 8;
    u8 y = 16;
    
    // Si 'flecha' es != 0 es que es la flecha derecha, cambiamos la coordenada X
    if (flecha) x = AREA_JUEGO_DER - x - SP_ARROW_LEFT_W;
    
    // Si 'iluminada' es cierto cambiamos los colores
    if (iluminada) {
        cambiaColorFlecha(15,13);
        cambiaColorFlecha(8,9);
    }
    // Si 'girada' es cierto volteamos el sprite para que mire a la derecha
    giraSpriteFlecha(girada);

    cpct_drawSpriteMaskedAlignedTable(sp_arrow_left,
        cpct_getScreenPtr(VMEM_BUFFER1, x, y),
        5, 12, sp_tablaTrans);

    // Si 'iluminada' es cierto volvemos cambiar los colores del sprite para dejarlos por defecto
    if (iluminada) {
        cambiaColorFlecha(13,15);
        cambiaColorFlecha(9,8);
    }
    
    // Volvemos a girarla para dejarla como estaba (mirando a la izquierda)
    giraSpriteFlecha(girada);
}

// ==============================================================
// Activa el pindado de las 2 flechas del menú selector de escenario
// ==============================================================
void pintaFlechasMenu() {
    pintarFlecha(0, NO, NO);
    pintarFlecha(1, SI, NO);
}


// ==============================================================
// Gestiona el menú de selector de escenario
//
// - DEVUELVE:
//      u8 selectedStage - El número de escenario seleccionado de 1 a 5, o 6 si es el random
// ==============================================================
u8 selectStage() {
    u8 pulsado = SI;
    u8 bufferFlechas = 0;

    borraPantallas();
    pintarMuro(0, 0, 4, 20);
    pintaFlechasMenu();

    pintarRecuadroDoble (20,  4, 40, 32);
    printTxt("SELECT STAGE", CENTRADO, 9, NO, 10);
    printTxt("FIRE TO PLAY", CENTRADO, 27, NO, 0);

    vaciaBufferTeclado();
    do {
        cpct_scanKeyboard();

        if (cpct_isKeyPressed(key[0].left) || cpct_isKeyPressed(key[1].left) || cpct_isKeyPressed(Joy0_Left) || cpct_isKeyPressed(Joy1_Left)) {
            sonidoTecla();
            pintarFlecha(0, NO, SI);
            selectedStage--;
            pulsado = SI;
        }
        if (cpct_isKeyPressed(key[0].right) || cpct_isKeyPressed(key[1].right) || cpct_isKeyPressed(Joy0_Right) || cpct_isKeyPressed(Joy1_Right)) {
            sonidoTecla();
            pintarFlecha(1, SI, SI);
            selectedStage++;
            pulsado = SI;
        }
        if (selectedStage > NUM_SELECT_STAGES) {
                selectedStage = 1;
        } else if (selectedStage == 0) {
                selectedStage = NUM_SELECT_STAGES;
        }
        if (cpct_isKeyPressed(Key_Esc)) {
            selectedStage = selectedStage * 10;
        }

       if (pulsado) {
            punteroMapa(NO);
            escribirNombreMapa();
            
            if (selectedStage == NUM_MAPA_RANDOM) {
                pintarMapaRandom();
            } else {
                descomprimir_pintarMapa();
            }
            vaciaBufferTeclado();
            pulsado = NO;
            bufferFlechas = 150;
        }

        if (bufferFlechas == 1) pintaFlechasMenu();
        if (bufferFlechas > 0) bufferFlechas--;
        
 
    } while (!cpct_isKeyPressed(key[0].fire) && !cpct_isKeyPressed(Joy0_Fire1) && !cpct_isKeyPressed(key[1].fire) && !cpct_isKeyPressed(Joy1_Fire1) && !cpct_isKeyPressed(Key_Esc));
    sonidoTecla();

    return selectedStage;
}

// ==============================================================
// REDIBUJAR LOS TEXTOS DEL MENÚ DE OPCIONES (PARA CUANDO SE HA CAMBIADO ALGUNA OPCIÓN)
// ==============================================================
void redibujaMenu() {
    pintaMenu();
    vaciaBufferTeclado();
}

// ==============================================================
// ALTERNAR ENTRE LAS DIFERENTES POSIBILIDADES DE LA OPCIÓN DE AUDIO
// ==============================================================
void cambiaOpcionesAudio() {
    if (bitOptions & mask_music_on) {
        if (bitOptions & mask_sfx_on) {
            bitOptions = bitOptions - mask_music_on;
            cpct_akp_musicInit((u8*) m_bewitch);
            reproducirMusica();
        } else {
            bitOptions = bitOptions | mask_sfx_on;
        }
    } else {
        if (bitOptions & mask_sfx_on) {
            bitOptions = bitOptions - mask_sfx_on;
        } else {
            bitOptions = bitOptions | mask_music_on;
            cpct_akp_musicInit((u8*) m_bewitch);
            reproducirMusica();
        }
    }
}

// ==============================================================
// ACTIVAR O DESACTIVAR EL BIT DE 'bitOptions'
// QUE ACTIVA O DESACTIVA LA CUENTA ATRÁS EN EL JUEGO
// SE DEFINE CON LA MÁSCARA: mask_enable_countdown
// ==============================================================
void cambiaOpcionesGameMode() {
    if (bitOptions & mask_enable_countdown) {
        bitOptions = bitOptions - mask_enable_countdown;
    } else {
        bitOptions = bitOptions | mask_enable_countdown;
    }
}

// ==============================================================
// SOLICITAR QUE PULSES TECLAS PARA REDEFINIRLAS
// Y ALMACENARLAS EN EL ARRAY key[]
// ==============================================================
void pedirTeclas() {
    u8 i;
    cpct_drawSolidBox((cpct_getScreenPtr(VMEM_BUFFER1, 24, 90)), 0, 38, 80);
    printTxt("PRESS", CENTRADO, 90, NO, 10);
    printTxt("PLAYER", 32, 100, NO, 13);
    for (i = 0; i < NUM_JUGADORES; i++) {
        printInt(i + 1, 46, 100, NO, 13);
        key[i].up    = redefineKey("UP");
        key[i].down  = redefineKey("DOWN");
        key[i].left  = redefineKey("LEFT");
        key[i].right = redefineKey("RIGHT");
        key[i].fire  = redefineKey("FIRE");
    }
}

// ==============================================================
// GESTIÓN GENERAL DEL MENÚ
// ==============================================================
void menu() {
    u16 semilla_random, tiempo_demo, tiempo_creditos;
    u8 n, opcion_mapa;
    u8 x_personaje = 1;
    u8 velocidad_personaje;
    u8 borrarCreditos = NO;
    u8 cambia_frame = 1;
    u8 frame_actual = 1;
    u8 *sprite;
    
    sprite = jugador[0].sprite_frame[frame_actual];
    n = opcion_mapa = semilla_random = tiempo_creditos = tiempo_demo = velocidad_personaje = 0;
 
    contador_musica = velocidad_musica = 5;
    cpct_akp_musicInit((u8*) m_bewitch);
    reproducirMusica();
 
    seleccionarMonoBufer();
    redibujaMenu();
  
    do {
        // Generar semillas random
        semilla_random++;
        if (!semilla_random) semilla_random = 1;
                
        // *************** CREDITOS
            // Rotar los creditos  
            if (tiempo_creditos == 0) {
                n = escribirCreditos(n);
                tiempo_creditos ++;
            } else {
                if (borrarCreditos == NO) {
                    if (!((tiempo_creditos % CAMBIAR_CADA) && tiempo_creditos != 0)) {
                        seleccionarSkin(0, SKIN_RANDOM);
                        borrarCreditos = x_personaje = SI;
                        velocidad_personaje = 0;
                    } 
                    if (++tiempo_creditos == NUM_CREDITOS * CAMBIAR_CADA) tiempo_creditos = 0;
                } else {
                    if (velocidad_personaje == 0) {
                        velocidad_personaje++;
                        if (x_personaje < 76) {
                            if (--cambia_frame == 0) {
                                cambia_frame = 5;
                                frame_actual = (frame_actual ? 0 : 1);
                                sprite = jugador[0].sprite_frame[frame_actual];
                            }
                            cpct_waitVSYNC();
                            cpct_drawSolidBox(cpct_getScreenPtr(VMEM_BUFFER1, x_personaje - 1, 169), 0, 4, SP_MAGO0_0_H+2);
                            cpct_drawSpriteMaskedAlignedTable(sprite,
                                cpct_getScreenPtr(VMEM_BUFFER1, x_personaje, 170), 
                                SP_MAGO0_0_W, SP_MAGO0_0_H, sp_tablaTrans);
                            x_personaje++;
                        } else {
                            //tiempo_creditos = 0;
                            borrarCreditos = NO;
                            cpct_drawSolidBox(cpct_getScreenPtr(VMEM_BUFFER1, x_personaje - 1, 170), 0, SP_MAGO0_0_W, SP_MAGO0_0_H);
                            n = escribirCreditos(n);
                        }
                    } else {
                        if (++velocidad_personaje > PAUSA_CAMINANDO) velocidad_personaje = 0;
                    }
                }
            }

        //  *************** FIN: CREDITOS

        // Rotar la demo
        if (borrarCreditos == NO) {
            if (++tiempo_demo == NUM_CREDITOS * CAMBIAR_CADA) {
                tiempo_demo = tiempo_creditos = 0;
                demo();
                redibujaMenu();
            }
        }

        // Control de tecla pulsada
        cpct_scanKeyboard();

        // (1) - INSTRUCTIONS
        if (cpct_isKeyPressed(Key_1) || cpct_isKeyPressed(Key_F1)) {
            sonidoTecla();
            tiempo_demo = tiempo_creditos = 0;
            instrucciones();
            borrarCreditos = NO;
            redibujaMenu();
        }
        
        // (2) - REDEFINE KEYS
        if (cpct_isKeyPressed(Key_2) || cpct_isKeyPressed(Key_F2)) {
            sonidoTecla();
            tiempo_demo = 0;
            pedirTeclas();
            pintaMenuOpciones();
        }

        // (3) - SOUNDS
        if (cpct_isKeyPressed(Key_3) || cpct_isKeyPressed(Key_F3)) {
            sonidoTecla();
            tiempo_demo = 0;
            cambiaOpcionesAudio();
            pintaMenuOpciones();
        }

        // (4) - GAME MODE
        if (cpct_isKeyPressed(Key_4) || cpct_isKeyPressed(Key_F4)) {
            sonidoTecla();
            tiempo_demo = 0;
            cambiaOpcionesGameMode();
            pintaMenuOpciones();
        }

        // (5) - START GAME
        if (cpct_isKeyPressed(Key_5) || cpct_isKeyPressed(Key_F5)) {
            sonidoTecla();
            opcion_mapa = selectStage();
            if (opcion_mapa > NUM_SELECT_STAGES + 1) {
                mapa_actual = opcion_mapa / 10;
                opcion_mapa = tiempo_demo = tiempo_creditos = 0;
                redibujaMenu();
            } else {
                mapa_actual = opcion_mapa;
            }
        }
    } while (!opcion_mapa);

    opcionMenu = SEGUIR_JUGANDO;
    cpct_srand(semilla_random);
}
